var kbar, i, c, R, omegabar, Rk, n, sigma, spread, credit, bankrupt, GDP, wedge, we;

varexo sigma_e sigma_w;

parameters sigma_ss mu gamma alpha delta beta rhosigma gam we_ss rhowe;

%dbstop in bgg_rbc_steadystate at 10
sigma_ss=0.26;
mu=0.21;
gamma=0.97;
alpha=1/3;
delta=0.02;
beta=1.03^(-.25);
rhosigma=0.97;
gam=1;
we_ss=.01;
rhowe=0.95;

model;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Auxiliary expressions.  These simplify the equations without adding
% additional variables.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  # F          = (normcdf(((log(omegabar) + sigma(-1)^2 / 2) / sigma(-1))));
  # G          = (normcdf((log(omegabar) + sigma(-1)^2 / 2) / sigma(-1) - sigma(-1)));
  # d          =  mu * G  * (1 + Rk) * kbar(-1);
  # Fp1        = (normcdf((log(omegabar(+1)) + sigma^2 / 2) / sigma));
  # Gp1        = (normcdf((log(omegabar(+1)) + sigma^2 / 2) / sigma - sigma));
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Equilibrium equations.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % resource constraint
    kbar(-1)^alpha = c + i + d ;
  
  % Law of motion for capital
    kbar = (1 - delta) * kbar(-1) + i;
  
  % Household FOC w.r.t. risk-free bonds
    1 = beta * (c(+1)/c)^(-gam)*(1+R);
  
  % efficiency condition associated with the contract
    0 = (1 - Gp1 - omegabar(+1) * (1 - Fp1)) * (1 + Rk(+1)) / (1 + R) + (1 - Fp1) / (1 - Fp1 - mu * omegabar(+1) 
        * normpdf((log(omegabar(+1)) + sigma^2 / 2) / sigma) / omegabar(+1) / sigma) * ((1 + Rk(+1)) / (1 + R) * ((1 - mu) * Gp1 
        + omegabar(+1) * (1 - Fp1)) - 1);
  
  % Definition of return of entrepreneurs, Rk
    1 + Rk = alpha*kbar(-1)^(alpha-1)  + 1 - delta;
    
  % Zero profit condition
    kbar(-1) * (1 + Rk) * ((1 - mu) * G + omegabar * (1 - F)) / (n(-1) * (1 + R(-1))) - kbar(-1) / n(-1) + 1;
  
  % Law of motion of net worth
    n = gamma*(1-G-omegabar*(1-F))*(1+Rk)*kbar(-1) + we;

  % borrowing:
    credit = kbar-n;

  % interest rate spread
    spread = ((1+Rk)/(1+R(-1)))*(kbar(-1)/credit(-1))*omegabar;

   % Shock 

   log(sigma / sigma_ss) = rhosigma * log(sigma(-1) / sigma_ss)  + sigma_e ;

   log(we / we_ss) = rhowe * log(we(-1) / we_ss) + sigma_w ;

   GDP = c + i;

   wedge = (1-(1+R)/(1+Rk(+1)));

   bankrupt = F;
end;

shocks;
var sigma_e; 
stderr .1;
var sigma_w;
stderr .1;
end;

steady;

stoch_simul(order=1,irf=20,nograph) kbar, i, c, R, omegabar, Rk, n, sigma, spread, credit, bankrupt, GDP, wedge;
ss=oo_.steady_state;
tt=0:length(kbar_sigma_e)-1;
ia=3;
ib=4;
subplot(ia,ib,12)
plot(tt,100*kbar_sigma_e/ss(1))
title('capital stock')
ylabel('% deviation from ss')
axis tight

subplot(ia,ib,1)
plot(tt,100*GDP_sigma_e/ss(12))
title('GDP (c + i)')
ylabel('% deviation from ss')
axis tight

subplot(ia,ib,2)
plot(tt,100*i_sigma_e/ss(2))
title('investment')
ylabel('% deviation from ss')
axis tight

subplot(ia,ib,3)
plot(tt,100*c_sigma_e/ss(3))
title('consumption')
ylabel('% deviation from ss')
axis tight

subplot(ia,ib,4)
plot(tt,400*R_sigma_e)
ylabel('dev from ss (APR)')
title('risk free rate')
axis tight

subplot(ia,ib,5)
plot(tt,400*spread_sigma_e)
ylabel('dev from ss (APR)')
title('interest rate spread')
axis tight

subplot(ia,ib,6)
plot(tt,400*Rk_sigma_e)
ylabel('dev from ss (APR)')
title('return on capital')
axis tight

subplot(ia,ib,7)
plot(tt,100*n_sigma_e/ss(7))
ylabel('% deviation from ss')
title('net worth')
axis tight

subplot(ia,ib,8)
plot(tt,wedge_sigma_e)
title('wedge, 1-(1+R)/(1+Rk(+1))')
ylabel('deviation from ss')
axis tight

subplot(ia,ib,9)
plot(tt,100*bankrupt_sigma_e)
ylabel('deviation *100')
title('bankruptcy rate')
axis tight

subplot(ia,ib,10)
plot(tt,100*credit_sigma_e/ss(10))
title('credit')
ylabel('deviation from ss')
axis tight

subplot(ia,ib,11)
plot(tt,100*sigma_sigma_e/ss(8))
title('sigma')
ylabel('% deviation from ss')
axis tight
