function [sk,sesk,sx,sesx,V]=skewm(x,thet,l,ip)
%
%This routine computes the skewness (sk) and standard deviation (sx)
%for a single time series, x, which is assumed to have a non-zero
%sample mean.  It also returns an estimate of the standard error
%of sk (sesk) and of sx (sesx).  The variance-covariance matrix
%of n^(1/2)*[mx,sx,sk] is given by V*n, where n is the number of
%observations in x.  V is computed using Lars Hansen's GMM formulas,
%and thet, l are parameters which have to be set to do this.
%Set thet = 1 and l = 1,2 or 3.  For more on these, see the writeup
%to MATLAB routine CORR.M.
%  The fact that the routine assumes the data are adjusted by the sample
%mean is important.  For example, when the data are iid and normal,
%then sesx = sqrt(6/n) when the sample mean is assumed to be
%estimated, while sesx = sqrt(15/n) when the sample mean is
%assumed to be known.
%  If ip = 1, the computations are done assuming the true value
%             of sk is zero.
%  If ip = 2, the computations are done assuming the true value
%             of sk is nonzero, and equal to its estimated value.
%
n=length(x);
mx=sum(x)/n;
sx=sqrt(sum((x-mx).^2)/n);
xx=sum((x-mx).^3)/n;
sk=xx/(sx^3);
X=zeros(n,3);
if ip == 1
D= [-1        0      0
    0       -2*sx    0
    3*sx^2    0     sx^3];
  for i=1:n
  X(i,:)=[x(i)-mx,(x(i)-mx)^2-sx^2,-(x(i)-mx)^3];
  end
end
if ip == 2
D= [-1        0        0
    0       -2*sx      0
    3*sx^2   (sx^2)*sk sx^3];
  for i=1:n
  X(i,:)=[x(i)-mx,(x(i)-mx)^2-sx^2,(x(i)-mx)^2*sk*sx-(x(i)-mx)^3];
  end
end
[V,S]=se(X,D,thet,l);
V=V/n;
sesk=sqrt(V(3,3));
sesx=sqrt(V(2,2));
