function [Yhat,Yhat_,VYhat] = forecastkalman1(Y,H,F,Q,R,B1,P,t1,LL,alf,initxi,tauvar,plot_levels,meanxxx,mm, ...
    trendm3,trendm1,fdiff,alpha,b,beta,is,taum,taub,taus,tausb,l_z,l_psi,D,taushock,outfolder,parstoch,date, ...
    nameshock,user,V_eps)

%--------------------------------------------------------------------------
%  Input:
%  The observed data are Y(:,t), t=1,...,T.
%  Observer equation: Y(:,t)=H*ksi(:,t)+w(:,t), w iid, Ew*w'=R.
%  State evolution equation: ksi(:,t+1)=F*ksi(:,t)+nu(:,t+1), nu iid, Enu*nu'=Q.
%  P unconditional variance of ksi (This is P1|0 in Hamilton's notation)

%  Output:
%  Yhat(:,t)=E[H*ksi(:,t)|information(t1)], for t=t1,...,t1+L,
%  where t1>=1, information(t1)={Y(:,1),...,Y(:,t1)}.
%  VAR(:,:,t) = variance of Yhat(:,t) forecast given information(t1), for t=t1,...,t1+L.
%  Yhat_= same as Yhat but enlarged to include all potentially observable
%  variables (i.e. not only the ones included in tauvar) and transformed in
%  level or y-o-y growth rates
%--------------------------------------------------------------------------

%Step 1: Obtain the Smoothed in-Sample Data, Yhat(:,1),...,Yhat(:,t1) and its
%        measure of uncertainty, VYhat(:,:,1),...,VYhat(:,:,t1)
%        xini=zeros(size(H,2),1);% = E[ksi(:,1)|no information]
%Step 2: Obtain out-of-sample point forecast and variance of forecast (the latter only in case we compute unconditional forecast)

conditional = 0; %=0 unconditional forecast
                 %=1 conditioning on one or more variables and the user provides string of innovations
                 %=2 conditioning on one or more variables and the user provides the value that constrained variables should assume and the location of shocks whose innovations are allowed to be non-zero
                 %=3 conditioning on one or more variables, using ECB Projections as baseline scenario, and the user provides the value that constrained variables should assume and the location of shocks whose innovations are allowed to be non-zero
                 %=4 same as =3 with alternative procedure
                 %=5 as 4 but it is done within a loop of one-step ahead at the time


T = size(Y,2);

l_xxx = size(taum,1);
if length(tauvar) < l_xxx
    if is == 1
        HH    =   [taum,taub,taus,tausb];
    else
        HH    =   [taum,taub,taus];
    end
else
    HH = H;
end

appendnomM  = 1; %=1 appends nominal M1 growth, nominal M3 growth, nominal oil price growth, nominal net-worth growth, nominal credit
if appendnomM == 1
    nstates = size(HH,2);
    addon = zeros(4,nstates);
    addon(1,:) = HH(2,:) + HH(8,:); % nominal M1 growth
    addon(2,:) = HH(2,:) + HH(9,:); % nominal M3 growth
    addon(3,:) = HH(2,:) + HH(1,:); % nominal net-worth growth
    addon(4,:) = HH(2,:) + HH(18,:); % nominal oil growth
    addon(5,:) = HH(2,:) + HH(4,:);  %nominal credit growth
    addon(6,:) = HH(2,:) + HH(20,:); %nominal reserve growth
    HH = [HH; addon];
    alf = [alf;0;0;0;0;0;0];
end

if user ==0
    if conditional == 0
        T_end = t1;
        [xsmooth, Vsmooth, VVsmooth, loglik] = kalman_smoother(Y(:,1:T_end), F, H, Q, R, initxi, P,alf(tauvar'));
    else
        T_end = T;
        %keyboard; T_end=t1;
        [xsmooth, Vsmooth, VVsmooth, loglik] = kalman_smoother(Y(:,1:T_end), F, H, Q, R, initxi, P,alf(tauvar'));
    end
elseif user ==1
    if conditional == 0
        T_end = t1;
        [xsmooth, Vsmooth, VVsmooth, loglik] = kalman_smoother(Y(:,1:T_end), F, H, Q, R,initxi, P);
    else
        T_end= T;
        [xsmooth, Vsmooth, VVsmooth, loglik] = kalman_smoother(Y(:,1:T_end), F, H, Q, R,initxi, P);        
    end
end

s = [2*l_z+1:2*l_z+l_psi];
for ii = 2:T_end
    err(:,ii) = xsmooth(s,ii)-F(s,s)*xsmooth(s,ii-1);
end
err(:,1) = xsmooth(s,1); 

%keyboard; err=[err zeros(46,13)];

[vbname,signal_on,II,III] = findnum(taushock,parstoch,l_psi,nameshock);

err = err(III,:);

err = inv(chol(V_eps))*err;

ninn = size(D,2);

innovation = zeros(ninn,LL); %innovations are set to zero

if conditional == 0

    whitening =0; %=1 use innovations that have been found fitting AR(1) process on each time-series innovation (estimated in sample); =0 do nothing, i.e. innovations are set to zero
    if whitening ==1
        figure; orient landscape
        n1=4;
        n2=4;
        tt_= t1-7 ; %fit AR(1) over last 8 qurters

        for ii=1:length(III)
            y_innov = err(ii,tt_:end)'; x_innov = err(ii,tt_-1:end-1)';
            bet_ = (x_innov'*x_innov)\(x_innov'*y_innov);

            innov_forcast(1) = y_innov(end)*bet_;
            for jk=2:LL
                innov_forcast(jk) = innov_forcast(jk-1)*bet_;
            end

            subplot(n1,n2,ii)
            plot(1:length(y_innov)+LL,[y_innov; innov_forcast'],'r-',1:length(y_innov)+LL,[y_innov; zeros(LL,1)],'k-')
            axis tight
            grid on
            innovation(ii,:) = innov_forcast;
        end
    end
end

if conditional ==1
    innovation_1 = zeros(ninn,LL); %err(:,t1+1:t1+LL); %

    %-----2000 scenario
    %         innovation_1([4,6,9,12:20],1:5) = err([4,6,9,12:20],find(date==2000.25):find(date==2001.25));
    %-----

    %----1998 scenario
    %         innovation_1([4,6,9,11,12:20],1:3) = err([4,6,9,11,12:20],find(date==1998.5):find(date==1999.0));

    %----early 1990s scenario
              innovation_1([9,12:20],1:9) = err([4,6,9,12:20],find(date==1990.5):find(date==1992.5)); %find(date==1990.75):find(date==1992.75)

    %------------
    innovation = innovation_1;
end
        
if conditional ==2 || conditional ==3 || conditional ==4 || conditional==5

    %load up the file containing the conditioning assumptions; used only for conditional=2,3,4
    %forecastkalman_input 
    %forecastkalman_input_ComparingModels
    %forecastkalman_input_WGFjune08
    %forecastkalman_input_SIP_june08
    %forecastkalman_input_WGFseptember08
    %forecastkalman_input_scenariopaper
    %forecastkalman_input_WGFdecember08
    forecastkalman_input_march09
    
    LLL = size(ybar,2);
    if LLL > LL
        error('(forecastkalman1) length of ybar containing values of conditioned variables is too long')
    end
end

if conditional ==2 
    innovation = innovation_counter;
end

if conditional ==3
    innovation = err(:,t1+1:t1+LL);

    innovation = innovation + innovation_counter; %innovation_counter is defined in forecastkalman_input.m
end

if conditional ==4 || conditional==5
    innovation_c = err(:,t1+1:t1+LLL);

    innovation_c = innovation_c + innovation_counter(:,1:LLL); %innovation_counter is defined in forecastkalman_input.m

    %if a row of innovation_counter >0 make sure that it does not belong to a shock in bb
    %    error('fatal (forkastkalman1) a row of innovation_counter is >0 and it belongs to a shock in bb')
    %end
end

%keyboard
BD = [B1*D; zeros(size(B1,1),ninn); D]*chol(V_eps);

[VYhat,Yhat,Yhat_] = forkalman(F,Q,xsmooth,Vsmooth,H,HH,R,alf,tauvar,BD,innovation,t1,LL);

Yhat_uncond = Yhat_;

if conditional ==2 || conditional ==3 || conditional ==4 || conditional==5

    [bbb,kk,nn,d] =  forecastkalman1_prepare(bb,ninn,taushock,parstoch,aa);
    
end

if conditional ==2 || conditional==3 || conditional==4

    Yhat_1 = Yhat_(aa,t1+1:t1+LLL)';
    Yhat_1 = reshape(Yhat_1,size(Yhat_1,1)*size(Yhat_1,2),1);

    rr = reshape(ybar',size(ybar,1)*size(ybar,2),1) - Yhat_1;

    if conditional ==2 && max(max(innovation_counter)) > 0 && kk > nn
        error('fatal (forkastkalman1) innovations of shocks that cannot adjust are different from zero & No of shocks > than No constrained variables. Use conditional=4')
    end
    if conditional ==3 && kk > nn
        %error('fatal (forkastkalman1) No of shocks > than No constrained variables and given that innovations that cannot adjust are different from zero, use conditional=4')
    end
end

if conditional == 2 || conditional == 3

    GG = zeros(LLL*nn,kk*LLL);

    for j=1:nn
        HBD = HH*BD;
        GG(1+(j-1)*LLL,1:kk) = HBD(aa(j),bbb);
        if LLL >= 2
            for i = 2:LLL
                HBD = HH*F^(i-1)*BD;
                GG(i+(j-1)*LLL,1:kk*i) = [HBD(aa(j),bbb)  GG(i-1+(j-1)*LLL,1:kk*(i-1)) ];
            end
        end
    end

    innovation_ = GG'/(GG*GG')*rr;  %GG'*inv(GG*GG')*rr;

    innovation_ = reshape(innovation_,kk,LLL);

    innovation_new =innovation;
    innovation_new(bbb,1:LLL) = innovation(bbb,1:LLL)+innovation_;
    [VYhat,Yhat,Yhat_] = forkalman(F,Q,xsmooth,Vsmooth,H,HH,R,alf,tauvar,BD,innovation_new,t1,LL); %we use here xsmooth_c, i.e. we assume that there is no uncertainty regarding the state of the economy

    rob=0;
    if rob==1
        LLL=LL;
        xx_inn = innovation_(1:LLL);
        optionals = optimset('LevenbergMarquardt','on','Display','iter','TolFun',1e-8,'TolX',1e-8,'MaxFun',300,'Diagnostics','on','MaxFunEvals',300);
        [xx_inn_new,fval,exitflag] = fminsearch(@findinnovations,xx_inn,optionals,F,Q,xsmooth,Vsmooth,H,HH, ...
            R,alf,tauvar,BD,innovation,t1,LL,bbb,Y,LLL);

        %option(4) = 30; %maximum number of iterations (15000 by default)
        %option(5) = 1;  %in order to print intermediate results
        %[xx_inn_new,fval,options] = solvopt(xx_inn,'findinnovations',[],option,[],[],F,Q,xsmooth_c,Vsmooth,H,HH, ...
        %                                          R,alf,tauvar,BD,innovation,t1,LL,bbb,Y,LLL);

        innovation_new(bbb,1:LLL) = innovation(bbb,1:LLL)+xx_inn_new;
        [VYhat,Yhat,Yhat_] = forkalman(F,Q,xsmooth,Vsmooth,H,HH,R,alf,tauvar,BD,innovation_new,t1,LL); %we use here xsmooth_c, i.e. we assume that there is no uncertainty regarding the state of the economy
    end
end

if conditional ==4

    GG = zeros(LLL*nn,ninn*LLL);

    for j=1:nn
        HBD = HH*BD;
        GG(1+(j-1)*LLL,1:ninn) = HBD(aa(j),:);
        if LLL >= 2
            for i = 2:LLL
                HBD = HH*F^(i-1)*BD;
                GG(i+(j-1)*LLL,1:ninn*i) = [HBD(aa(j),:)  GG(i-1+(j-1)*LLL,1:ninn*(i-1)) ];
            end
        end
    end

    % --rearrange shocks by creating selection matrix to say which shocks are constrained
    cinn = ninn - kk;
    Smat = zeros(cinn,ninn);
    ccc  = find(d==0);

    for jkl = 1:cinn
        Smat(jkl,ccc(jkl)) = 1;
    end
    Smat = Smat';
    Smat = kron(eye(LLL), Smat);
    GS = GG*Smat;

    % -- also create selection matrix for unconstrained shocks
    Stildemat = zeros(kk,ninn);
    for jkl = 1:kk
        Stildemat(jkl,bbb(jkl)) = 1;
    end
    Stildemat = Stildemat';
    Stildemat = kron(eye(LLL), Stildemat);
    GStilde = GG*Stildemat;

    % -- set up Gtilde
    Gtilde = [GS,GStilde];
    sbar = [];
    for jkl = 1:LLL
        sbar = [sbar; innovation_c(ccc,jkl)];
    end

    % -- max-likelihood value for shocks given constraints
    innovation_ = GStilde'/(GStilde*GStilde')*(rr-GS*sbar);  %GStilde'*inv(GStilde*GStilde')*(rr-GS*sbar);
    innovation_ = reshape(innovation_,kk,LLL);

    % --replace the unconstrained innovations in innovation-vector
    innovation_new =innovation_c;

    innovation_new(bbb,1:LLL) = innovation_;

    [VYhat,Yhat,Yhat_] = forkalman(F,Q,xsmooth,Vsmooth,H,HH,R,alf,tauvar,BD,innovation_new,t1,LL); %we use here xsmooth_c, i.e. we assume that there is no uncertainty regarding the state of the economy
end


if conditional ==5

    Yhat_c = Yhat_uncond;
    xsmooth_c = xsmooth;
    Vsmooth_c = Vsmooth;

    [GS,GStilde,ccc] = forecastkalman1_cond5(nn,ninn,HH,BD,aa,bbb,kk,d);

    for LLL_1 = 1:LLL

        Yhat_1 = Yhat_c(aa,t1+LLL_1)';
        Yhat_1 = reshape(Yhat_1,size(Yhat_1,1)*size(Yhat_1,2),1);

        rr = reshape(ybar(:,LLL_1)',size(ybar,1)*1,1) - Yhat_1;

        sbar = innovation_c(ccc,LLL_1);

        % -- max-likelihood value for shocks given constraints
        innovation_ = GStilde'/(GStilde*GStilde')*(rr-GS*sbar);  %GStilde'*inv(GStilde*GStilde')*(rr-GS*sbar);
        innovation_ = reshape(innovation_,kk,1);

        % --replace the unconstrained innovations in innovation-vector

        innovation_c(bbb,LLL_1) = innovation_;
        innovation_new = [innovation_c(:,LLL_1) zeros(size(innovation_c,1),1)];

        [VYhat,Yhat,Yhat_c,xsmooth_c,Vsmooth_c] = forkalman(F,Q,xsmooth_c,Vsmooth_c,H,HH,R,alf,tauvar,BD,innovation_new,t1+(LLL_1-1),2);
    end

    %now go up to LL given that LL can be > LLL (if LL=LLL nothing will change)
    innovation_c = [innovation_c innovation(:,LLL+1:LL)];
    [VYhat,Yhat,Yhat_,xsmooth_n,Vsmooth_n] = forkalman(F,Q,xsmooth_c,Vsmooth_c,H,HH,R,alf,tauvar,BD,innovation_c,t1,LL);

end


%------- the following code converts the variables in y-o-y changes or levels
plot_data_mean= 2;
idat=0; %set =1 when prepare_short below works with the actual data. In that case, we want to do some smoothing on consumption before computing the marginal utility of consumption. Otherwise mu will go negative
if plot_data_mean==1  %remove estimated mean and add mean of the data
    Yhat_ = Yhat_ - mean((Yhat_(1:l_xxx,:))')'*ones(1,size(Yhat_,2)) + meanxxx*ones(1,size(Yhat_,2));
elseif plot_data_mean==2
    Yhat_ = Yhat_ - alf*ones(1,size(Yhat_,2));
end
trendm1=zeros(size(Yhat_,2),1); %set trend to zero to avoid adding up the trend n times in the analysis of shocks below, where n is the number of shocks
trendm3=zeros(size(Yhat_,2),1);
[laborwedge,laborwedgea,effwedge,net,dmoney3,dmoney1,dinvest,dout,infl,dwage,dcons,Spread1,money1,money3, ...
    output,invest,cons,wage,price,hours,capacity,btott,dbtott,Rm_series,Re_series,Pe_series,govc,dgovc,idef,didef, ...
    oilp,doilp,Spread2,dnet,reserves,dreserves] ...
    = prepare_short(Yhat_,mm,trendm3,trendm1,size(Yhat_,2),fdiff,alpha,b,beta,idat);

if plot_levels == 1 %levels
    XXXt = [net;price;hours;btott;output;wage;invest;money1;money3;cons;Pe_series;Spread1;Rm_series;Re_series;capacity;govc;idef;oilp;Spread2;reserves];
elseif plot_levels == 0 %leave variables as they are
    XXXt = Yhat_;
elseif plot_levels == 2 %year-on-year growth rates
    XXXt = [dnet;infl;hours;dbtott;dout;dwage;dinvest;dmoney1;dmoney3;dcons;Pe_series;Spread1;Rm_series;Re_series;capacity;dgovc;didef;doilp;Spread2;dreserves];
end

Yhat_ = XXXt;
