function [] = plot_contributions(decomp_new,timevec,startplot,tauvar,XXXt,lgy_,check1,vert_bar,cmap,lgx_,str)

% - plot options
Fontsizex = 15;   % Fontsize for the plots %25
stepsize  = 1;    % stepsize = 1: print a bar for every date. stepsize = 2: for every second date, ...
steptick  = 4;    % plot the label with date every steptick-th bar only
if size(XXXt,2)-startplot >50
  steptick  = 8;   
end

% -------------------------------------------------------------------------
XXXt      = XXXt';
ncat      = size(lgx_,1); % # of categories of shocks
nvar      = length(tauvar);
nobs      = length(timevec);

% --------------------------------------------------
% For each variable plot stacked bars
% --------------------------------------------------
for klm = 1:nvar  
    % -- check that the categories of shocks have been correctly specified
    checker   = [];
    checker = decomp_new(:,ncat*(klm-1)+[1:ncat]); % counterfactuals without deterministics  
    if abs(max(max(sum(checker,2)-(check1(klm,:))'))) > 0.1e-10
        error('fatal (plot_contributions) the sum of shocks categories does not add up')
    end
    
    % -- create positive and negative entries separately
    checkmatp  = zeros(size(checker));
    checkmatn  = zeros(size(checker));

    for jkl = 1:nobs;   % loop over observations
        checkvec  = checker(jkl,:);
        indexn    = find(checkvec<0);
        indexp    = find(checkvec>=0);
        checkvecp = checkvec(indexp);
        checkvecn = checkvec(indexn);   
        checkmatp(jkl,indexp) = checkvecp;
        checkmatn(jkl,indexn) = checkvecn;
    end
    
    % -- do plot
    str_ = ['Contribution to ',deblank(lgy_(klm,:)), '. ', str];
    figure('name',str_); orient landscape
    scrsz = get(0,'ScreenSize');
    %figure('Position',[1 1 scrsz(3:4)/1.05]);
    
    %figure('Position',[1 scrsz(4)/1.05 scrsz(3)/1.05 scrsz(4)/1.05])
    hold on 
    bar(startplot:stepsize:nobs,checkmatp(startplot:stepsize:nobs,:),0.5,'stacked') %,'EdgeColor','flat') %0.5
    colormap(cmap)
    bar(startplot:stepsize:nobs,checkmatn(startplot:stepsize:nobs,:),0.5,'stacked') %,'EdgeColor','flat') %0.5
    colormap(cmap)
    set(gca, 'Fontsize',Fontsizex-3)
    plot(startplot:stepsize:nobs, XXXt(startplot:stepsize:nobs,klm), 'LineWidth',2,  'Color', [0 0 0], 'Marker', 'o', 'MarkerSize',2,'MarkerFaceColor',[1 0.75 0.25],'MarkerEdgeColor',[0,0,0])  %[1 0.5 0],  
    set(gca, 'Xtick',startplot:steptick*stepsize:nobs);
    set(gca, 'Xticklabel',timevec(startplot:steptick*stepsize:nobs))

    title(str_ ,'Fontsize', Fontsizex+4, 'VerticalAlignment', 'bottom')
    line('XData',[startplot-1, nobs+1],'YData',[0, 0], 'LineStyle','-', 'LineWidth', 2)
    if isempty(vert_bar) ~=1
       line('XData',[find(timevec==vert_bar)+0.5, find(timevec==vert_bar)+.5],'YData',[min(min([sum(checkmatn(startplot:stepsize:nobs,:),2), XXXt(startplot:stepsize:nobs,klm)])), ...
                                                       max(max([sum(checkmatp(startplot:stepsize:nobs,:),2), XXXt(startplot:stepsize:nobs,klm)]))], ...
                                                       'LineStyle','-', 'LineWidth', 3, 'Color',[0.3 0.3 0.3])
    end
    
    hold off
    grid on
    leg = legend(lgx_(:,:));
    set(leg, 'FontSize', Fontsizex-6,'Location','SouthEast');

    axis tight 
    orient landscape
    
    % -- print
    %keyboard
    %nameit = horzcat('contrib_var',num2str(klm), '.emf');
    %nameit = horzcat('print -dmeta ', nameit);
    %eval(nameit)    
    
end

% - legend figure
% figure;     figure('Position',[1 1 scrsz(3:4)/1.05]);
% bar(1:10,checkmatn(1:10,:)*0,'stacked')
% colormap(cmap)
% axis([0 1 0.5 1])
% leg = legend(lgx_(:,:));
% set(gca, 'Xtick',[]);
% set(gca, 'Ytick',[]);
% set(leg, 'FontSize', Fontsizex);
% hold off
% axis tight
% nameit = horzcat('contrib_legend.emf');
% nameit = horzcat('print -dmeta ', nameit);
% eval(nameit)
% close all