function [Yhat] = varforecast(nT,nlags,Xdat,hasconstant,exogvar)

%Input:
% nT
% nlags
% Xdat(:,1),...,Xdat(:,T)
% hasconstant: optional; =1 if you want constant to be included; 0 otherwise; default=1
% exogvar: optional; exogenous variables; default=[]

%Output:
%Yhat(:,1),...,Yhat(:,T), where Yhat(:,t)=Xdat(:,t) for t=1,...,nT
% and Yhat(:,nT+s), s=1,...,T-nT is forecast of Xdat(:,nT+s) given Yhat(:,t), t=1,...,nT. 
% Forecast is computed with nlags VAR fit to Xdat(:,1),...,Xdat(:,nT).
% 
% 

if nargin==3
    hasconstant=1;
    exogvar=[];
elseif nargin==4
    exogvar=[];
end
[varcoef,VV,uz]=estimatevar(Xdat(:,1:nT)',nlags,hasconstant,exogvar);
if hasconstant==1
    AA=varcoef(:,[2:end]);
    const=varcoef(:,1);
else
    AA=varcoef(:,[1:end]);
    const=zeros(size(varcoef,1),1);
end
[rrts] = companion(nlags,AA);
if max(abs(rrts)) > 1
    disp('warning! Explosive Eigenvalue in VAR')
end

nn=size(Xdat,1);
T=size(Xdat,2);
Yhat=zeros(nn,T);
resid=zeros(nn,nT);
resid(:,[nlags+1:nT])=uz';

Yhat(:,[1:nlags])=Xdat(:,[1:nlags]);

for ii = nlags+1:nT
    
    if ii <= nT
        Yhat(:,ii)=const+resid(:,ii);
    else
        Yhat(:,ii)=const;
    end
    
    for k = 1:nlags
        Yhat(:,ii)=Yhat(:,ii)+AA(:,[(k-1)*nn+1:k*nn])*Yhat(:,ii-k);
    end
end

for ii = nT+1:T
    Yhat(:,ii)=const;
    for k = 1:nlags
        Yhat(:,ii)=Yhat(:,ii)+AA(:,[(k-1)*nn+1:k*nn])*Yhat(:,ii-k);
    end
end