// following are the 'endogenous' variables:
var lc, lh, lk, ly, z, li, Rk;
// following are the shocks
varexo e;

parameters beta, rho, alpha, delta, psi, util, css, hss, kss, yss, zss;

alpha = 0.36;
rho   = 0.9999;
beta  = 0.97;
delta = .1;
psi   = 2.5;

[cssp,hssp,kssp,yssp,zssp] = ssrbc(alpha, beta, delta, psi, rho);
css=cssp;
hss=hssp;
kss=kssp;
yss=yssp;
zss=zssp;

model;
    1/exp(lc) = beta/exp(lc(+1))*(alpha*exp(ly(+1)-lk) + 1 - delta);
    exp(lc)*psi/(1-exp(lh)) = (1-alpha)*exp(ly-lh);
    exp(ly) = exp(lc) + exp(lk) - (1-delta)*exp(lk(-1));
    exp(ly) = exp(alpha*lk(-1))*((z*exp(lh))^(1-alpha)); 
    log(z) = rho*log(z(-1)) + e;
    exp(li) = exp(lk) - (1-delta)*exp(lk(-1));
    Rk = alpha*(z(+1)*exp(lh(+1)-lk))^(1-alpha) + 1-delta;
end;

initval;
lc=log(css);
lh=log(hss);
lk=log(kss);
ly=log(yss);
z=zss;
li=log(delta*kss);
Rk=alpha*(zss*hss/kss)^(1-alpha)+1-delta;
end;

steady;

shocks;
// the standard error here is ridiculously high....100%. This is so that irf's will be in percent terms.
var e; 
stderr 1.;
end;

stoch_simul(order=1,irf=20) lc lh lk ly z li Rk;

// here is code that answers the various parts of question 1....
fprintf('in this output, what is referred to as ''percent'' actually has not been multiplied by 100');
fprintf('1a response of output = %10.7f, predicted response of output = %10.7f\n',ly_e(1),(1-alpha)*(lh_e(1)+z_e(1)));
fprintf('1b percent response in the consumption-output ratio, c/y, = %5.2f\n',lc_e(1)-ly_e(1));
dyhat=ly_e;
dchat=lc_e;
dihat=(dyhat-(css/yss)*dchat)/(delta*kss/yss);
fprintf('1c&d vector of impulse response of log investment\n');
fprintf(' based on formula in question, based on investment computed in model, difference');
[dihat li_e dihat-li_e]
dcdy=(lc_e(1)/ly_e(1))*css/yss;
didy=(li_e(1)/ly_e(1))*delta*kss/yss;
fprintf('1e change(c)/change(y) = %5.2f, change(i)/change(y) = %5.2f\n',dcdy,didy);
fprintf('1g the net return on capital in the period of the shock (APR) = %5.2f, the return in steady state = %5.2f\n',100*(Rk_e(1)+1/beta-1),100*(1/beta-1));